package ru.ifmo.ga;

public class BitstringConfiguration extends Configuration<IntString> {
    protected Crossover<IntString> createCrossover() {
        return new Crossover<IntString>() {
            private final static double RATE = 0.9;

            public void crossover(IntString p1, IntString p2, IntString c) {
                if (Math.random() > RATE) {
                    c.copy(Math.random() > 0.5 ? p1 : p2);
                    return;
                }
                int pos = (int) (Math.random() * c.length());
                for (int i = 0; i < c.length(); i++) {
                    c.set(i, i <= pos ? p1.get(i) : p2.get(i));
                }
            }
        };
    }

    protected Mutation<IntString> createMutation() {
        return new Mutation<IntString>() {
            private final static double RATE = 0.1;   // percentage of bits to change

            public void mutate(IntString c) {
                final double prob = RATE / c.length();
                for (int i = 0; i < c.length(); i++) {
                    if (Math.random() < prob) {
                        c.set(i, 1 - c.get(i));
                    }
                }
            }
        };
    }

    protected Randomizer<IntString> createRandomizer() {
        return new Randomizer<IntString>() {
            public IntString randomize(int length) {
                IntString ch = new IntString(length);
                for (int i = 0; i < length; i++) {
                    ch.set(i, Math.random() > 0.5 ? 1 : 0);
                }
                return ch;
            }
        };
    }

    protected Selector createSelector() {
        return SelectorFactory.getInstance().rouletteSelector();
    }

    protected Utils<IntString> createUtils() {
        return new Utils<IntString>() {
            public void copy(IntString src, IntString dst) {
                dst.copy(src);
            }

            public double diff(IntString c1, IntString c2) {
                return 0;
            }

            public IntString allocate(int length) {
                return new IntString(length);
            }
        };
    }


}
