package ru.ifmo.hmm;

import ru.ifmo.ga.*;

public class Configuration extends ru.ifmo.ga.Configuration<Topo> {
    private final int N;

    protected Crossover<Topo> createCrossover() {
        return new Crossover<Topo>() {
            private final static double RATE = 0.3;

            public void crossover(Topo p1, Topo p2, Topo c) {
                if (Math.random() > RATE) {
                    c.copy(Math.random() > 0.5 ? p1 : p2);
                    return;
                }

                int p = (int) (Math.random() * p1.f.length);
                for (int i = 0; i < p; i++) {
                    int[] src = i < p ? p1.f[i] : p2.f[i];
                    System.arraycopy(src, 0, c.f[i], 0, src.length);
                }
            }

        };

    }

    protected Mutation<Topo> createMutation() {
        return new Mutation<Topo>() {

            private final static double RATE = 0.1;

            public void mutate(Topo c) {
                for (int f[] : c.f) {
                    mutate(f);
                }
            }

            private void mutate(int[] f) {
                for (int i = 0; i < f.length; i++) {
                    if (Math.random() < RATE) {
                        f[i] = rnd();
                    }
                }
            }

        };
    }

    private int rnd() {
        return (int) (Math.random() * N);
    }

    protected Randomizer<Topo> createRandomizer() {
        return new Randomizer<Topo>() {
            private void randomize(int[] f) {
                for (int i = 0; i < f.length; i++) {
                    f[i] = rnd();
                }
            }

            public Topo randomize(int length) {
                Topo c = new Topo(N, length);
                for (int s[] : c.f) {
                    randomize(s);
                }
                return c;
            }
        };
    }

    protected Selector createSelector() {
        return SelectorFactory.getInstance().rankSelector();
    }

    protected Utils<Topo> createUtils() {
        return new Utils<Topo>() {
            private double diff(int x[], int y[]) {
                int d = 0;
                for (int xx : x) {
                    for (int yy : y) {
                        if (xx != yy) d++;
                    }
                }
                return d * 1.0 / x.length;
            }

            public void copy(Topo src, Topo dst) {
                dst.copy(src);
            }

            public double diff(Topo c1, Topo c2) {
                double d = 0;
                for (int i = 0; i < c1.f.length; i++) {
                    d += diff(c1.f[i], c2.f[i]);
                }
                return d / c1.f.length;
            }

            public Topo allocate(int length) {
                return new Topo(N, length);
            }
        };
    }

    public Configuration(int n) {
        this.N = n;
    }

    protected double getFittestModified(double v) {
        return -1 / v;
    }

}
