package ru.ifmo.hmm;

import ru.ifmo.ga.EvalChromosome;
import ru.ifmo.ga.Genome;
import org.apache.log4j.Logger;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;

import static ru.ifmo.hmm.DataProvider.*;

public class Main {

    private final static Logger LOG = Logger.getLogger(Main.class);

    private static final int POPULATION_SIZE = 100;
    private static final int POPULATIONS = TRIES / POPULATION_SIZE;

    private static final double EPS = 1e-8;
    private static final boolean CHECK_NULLS = true;

    private static double oldA[][];

    public static void main(String[] args) {
        LOG.info("Pop size: " + POPULATION_SIZE);
        
        oldA = new double[N][N]; // aux

        ru.ifmo.ga.Configuration<Topo> cfg = new Configuration(N);
        cfg.setPopulationSize(POPULATION_SIZE);
        cfg.setChromosomeLength(KA);
        cfg.setFitness(new Fitness());
        Genome<Topo> genome = Genome.randomize(cfg);
        double est3 = genome.getFittest();
        for (int t = 0; t < POPULATIONS; t++) {
            genome.evolve();
        }
        
        LOG.info("best: " + est3);
    }


    private static class Fitness implements ru.ifmo.ga.Fitness<Topo> {
        private final Hmm hmm;

        private final double b[][] = new double[N][M];
        private final double p[] = new double[N];

        private Fitness() {
            hmm = new Hmm(new double[N][N], b, p);
        }

        private void copy() {
            for (int i = 0; i < N; i++) {
                System.arraycopy(hmm1.b[i], 0, b[i], 0, M);
            }
            System.arraycopy(hmm1.h, 0, p, 0, N);
        }

        public void fitness(EvalChromosome<Topo>[] pop) {
            for (EvalChromosome<Topo> c : pop) {
                c.v = fitness(c.c);
            }
        }

        private double fitness(Topo c) {
            copy();
            rememberGraph(c.f);
            hmm.rndQuasidetermA(DA, c.f);
            checkGraph();
            return -1 / hmm.solve3(TRAIN, 1);

        }


        private void checkGraph() {
            if (!CHECK_NULLS) return;

            for (int i = 0; i < N; i++) {
                for (int j = 0; j < N; j++) {
                    boolean z1 = Math.abs(oldA[i][j]) > EPS;
                    boolean z2 = Math.abs(hmm.a[i][j]) > EPS;
                    if (z1 ^ z2) {
                        System.out.println(i + ", " + j + ": " + oldA[i][j] + " -> " + hmm.a[i][j]);
                    }
                }
            }

        }

        private void rememberGraph(int[][] f) {
            if (!CHECK_NULLS) return;

            for (int i = 0; i < N; i++) {
                Arrays.fill(oldA[i], 0);
            }
            for (int i = 0; i < N; i++) {
                oldA[i][(i + 1) % N] = 1;
            }

            for (int i = 0; i < N; i++) {
                for (int k = 0; k < f[i].length; k++) {
                    oldA[i][f[i][k]] = 1;
                }
            }
        }

    }

    private static void save(String fileName) throws IOException {
        PrintWriter pw = new PrintWriter(new FileWriter(fileName));
        pw.println(SAMPLES);
        for (int[] t : TRAIN) {
            for (int i : t) {
                pw.print(i + " ");
            }
            pw.println();
        }
        pw.close();
    }

}