package ru.ifmo.hmm;

import static ru.ifmo.hmm.DataProvider.*;

public class RandomSearch {

    private static void copy(Hmm hmm1, double b[][], double p[]) {
        for (int i = 0; i < N; i++) {
            System.arraycopy(hmm1.b[i], 0, b[i], 0, M);
        }
        System.arraycopy(hmm1.h, 0, p, 0, N);
    }

    private static void stepByStep() {
        hmm1.rndQuasideterm(DA, KA, DB, KB, DP, KP);
        hmm1.updateSampling(true);
        for (int i = 0; i < SAMPLES; i++) {
            TRAIN[i] = hmm1.generate(LEN);
        }
        double est1 = hmm1.solve1Ln(TRAIN);
        System.out.println("original: " + est1);

        double est2 = hmm1.solve3(TRAIN, 1);
        System.out.println("optimized: " + est2);

        final double b[][] = new double[N][M];
        final double p[] = new double[N];
        Hmm hmm2 = new Hmm(new double[N][N], b, p);

        int d[][] = new int[N][KA];
        for (int i = 0; i < N; i++) {
            for (int j = 0; j < KA; j++) {
                d[i][j] = (int) (Math.random() * N);
            }
        }

        int curr[] = new int[KA];
        double max = Double.NEGATIVE_INFINITY;
        for (int i = 0; i < N; i++) {
            System.arraycopy(d[i], 0, curr, 0, KA);
            for (int j = 0; j < N; j++) {
                for (int k = 0; k < j; k++) {
                    copy(hmm1, b, p);
                    d[i][0] = j;
                    d[i][1] = k;
                    hmm2.rndQuasidetermA(DA, d);
                    double v = hmm2.solve3(TRAIN, 1);
                    if (v > max) {
                        max = v;
                        System.out.println("max: " + max);
                    } else {
                        System.arraycopy(curr, 0, d[i], 0, KA);
                    }
                }
            }
        }
    }

    private static void random() {
        double best = Double.NEGATIVE_INFINITY;
        String bestHmm = "";
        int pos[][] = new int[hmm1.n][2];

        for (int i = 0; i < TRIES; i++) {

            // train it
            Hmm hmm2 = hmm1.copy();
            for (int j = 0; j < pos.length; j++) {
                for (int k = 0; k < pos[j].length; k++) {
                    pos[j][k] = (int) (Math.random() * hmm1.n);
                }
            }
            hmm2.rndQuasidetermA(DA, pos);
            hmm2.estimate(TRAIN, 1);

            double err = hmm2.solve3(TRAIN, 1);

            // update best
            if (best < err) {
                best = err;
                double r = best;

                System.out.println((i + 1) * 100.0 / TRIES + "%\t" + r);

                bestHmm = hmm2.toString();
            }


        }
        System.out.println(bestHmm);

    }

    private static double[] step(Hmm hmm1, double max) {

        hmm1.randomizeTopology(100);

        // generate C-element train set with seq len T
        int train[][] = new int[SAMPLES][LEN];
        for (int i = 0; i < SAMPLES; i++) {
            train[i] = hmm1.generate(LEN);
        }

        // how good hmm1 on it's own test
        double orig = hmm1.solve1Ln(train);

        // create ru.ifmo.hmm
        Hmm hmm2 = new Hmm(hmm1.n, hmm1.m);

        double best = hmm2.solve3(train, 1);

        for (int i = 0; i < TRIES; i++) {

            if ((best - orig) / orig < max) {
                break;
            }

            // radnomize it somehow
//            hmm2.rndUniform();
//            hmm2.rndQuasideterm(0.1, 2, 0.1, -1, 0.1, -1);
            hmm2.randomize(0.1);

            // train it
            double err = hmm2.solve3(train, 1);

            // update best
            best = Math.max(best, err);

        }

        return new double[]{(best - orig) / orig, orig, best};
    }

    public static void main(String[] args) {
//        m1();
//        m3();
        random();
//        stepByStep();
    }

    private static int[] fromString(String s) {
        int x[] = new int[s.length()];
        for (int i = 0; i < x.length; i++) {
            x[i] = s.charAt(i) - '0';
        }
        return x;
    }

    private static void m1() {
        Hmm hmm = new Hmm(4, 2);
        String best = "";
        double[] max = {Double.NEGATIVE_INFINITY, 0, 0};
        for (int i = 0; i < 1000000; i++) {
            double[] v = step(hmm, max[0]);
            if (v[0] > max[0]) {
                max = v;
                best = hmm.toString();
                System.out.println(max[1] + "\n" + max[2] + "\n" + "res: " + max[0] * 100 + "\n");
                System.out.println(best);
                System.out.println("---------------------------------------------------");

            }
        }

        System.out.println(max[1] + "\n" + max[2] + "\n" + "res: " + max[0] * 100 + "\n");
        System.out.println(best);
    }


}
