package newCynic;

import ru.ifmo.robocode.TankManager;
import robocode.Bullet;

/**
 *  "".
 */

public class Gunner  {

    /**  . */
    private double _curHeading;
    /**   . */
    private double _curGunHeat; //  .
    /**   . */
    private double _gunHeatDecrement;
    /**  .*/
    private Target _curTarget;
    /**  .*/
    private GeomVector _curAim;
    /**   .*/
    private double _curFirepower;
    /**       . */
    private double _da;
    /**  . */
    private double _firepower;
    /** - */
    private Cynical _robot;

    /** .
     *   ""   -.
     * @param aRobot 
     */
    Gunner(Cynical aRobot) {
        _robot = aRobot;
        GunnerState.reset(this);
        _gunHeatDecrement = _robot.getGunCoolingRate();
        System.out.println("New: " + _gunHeatDecrement);
        _curAim = new GeomVector();

    }

    /** ,     . */
    public void beginRound() {
        _curHeading = _robot.getGunHeadingRadians();
        _curGunHeat = _robot.getGunHeat();
        _curTarget = null;
        _curAim.reset();
        _robot._lastShotTime = 0;
    }
    /**  .*/
    public void beginTurn() {
        _curHeading = _robot.getGunHeadingRadians();
        _curGunHeat = _robot.getGunHeat();

        _da = 0;
        _firepower = 0;

        GunnerState.processIncomingEvent(10, this);
    }

    /**  . */
    public void endTurn() {
        _robot.setTurnGunRightRadians(_da);
        if (_firepower >= 0.1) {
            TankManager.getInstance().fire();
            Bullet bullet = _robot.fireBullet(_firepower);
            if (bullet != null) {
                _robot._lastShotTime = _robot._currentTime;
                if (_curTarget != null) {
                    _curTarget.shoot(_firepower);
                }
            }
        } else {
            _robot.scan();
        }
    }

    /**   . */
    public double getCurHeading() {
        return _curHeading;
    }

    /**  */
    private GunnerState _state;

    /**
     *   .
     * @return ,   
     */
    public GunnerState getState() {
        return _state;
    }

    /**
     *   
     * @param aState  
     */
    public void setState(GunnerState aState) {
        _state = aState;
    }
/*
      .
*/
    /** x20 :   (  3 ) . */
    public boolean x20_gunIsExpectedToBeCold() {
        boolean result = _curGunHeat / _gunHeatDecrement <= 3;
        if (Constants.INPUTS_LOGGING) {
            Logger.logInput("x20", "  ", result);
        }
        return result;
    }

    /** x21 :  . */
    public boolean x21_gunIsCold() {
        boolean result = _curGunHeat <= 0;
        if (Constants.INPUTS_LOGGING) {
            Logger.logInput("x21", " ", result);
        }
        return result;
    }

    /** x22 :       . */
    public boolean x22_gunWillBeColdWithinTwoSteps() {
        boolean result = _curGunHeat / _gunHeatDecrement <= 1;

        if (Constants.INPUTS_LOGGING)
            Logger.logInput("x22", "      ", result);
        return result;
    }

    /** x25 :  . */
    public boolean x25_targetIsCaptured() {
        boolean result = _curTarget != null;

        if (Constants.INPUTS_LOGGING)
            Logger.logInput("x25", " ", result);
        return result;
    }

    /** x26 :  . */
    public boolean x26_targetIsLost() {
        boolean result = true;

        if (_curTarget != null)
            result = !_curTarget.isTracked();

        if (Constants.INPUTS_LOGGING)
            Logger.logInput("x26", " ", result);
        return result;
    }

    /** x30 :       . */
    public boolean x30_gunWillTurnWithinTwoSteps() {
        boolean result = true;
        double gun_to_go = Cynical.getAngleDiff(_curHeading, _curAim.getAngle());
        double turn_direction = gun_to_go >= 0 ? 1 : -1;
        double gun_turning_speed = turn_direction * Cynical.MAX_GUN_ROTATION_SPEED
                + _robot.getDriver().getTurningSpeed();

        result = Math.abs(gun_to_go / gun_turning_speed) <= 1;
        if (Constants.INPUTS_LOGGING)
            Logger.logInput("x30", "      ", result);
        return result;
    }

   /** x50 :  .*/
    public boolean x50_isPointingFine() {
        boolean result = true;

        double gun_to_go = Cynical.getAngleDiff(_curHeading, _curAim.getAngle()); //shortest_turn( cur_aim.a - getCurHeading ) ;
        result = Math.abs(gun_to_go) < Cynical.PRECISION;

        if (Constants.INPUTS_LOGGING)
            Logger.logInput("x50", " ", result);
        return result;
    }
/*
      .
*/
    /** z30 :  .*/
    public void z30_selectTarget() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z30", " ");
        }

        _curTarget = _robot.getTargets().getClosestTarget(8);
        if (_curTarget == null)
            _curTarget = _robot.getTargets()._closestTarget;
    }

    /** z40 :   . */
    public void z40_calculateFirePower() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z40", "  ");
        }

        double P = 0.25;
        if (_robot._currentEnegy >= Cynical.ENERGY_NORMAL_THRESHOLD) {
            P = 0.2;
            if (_robot._aliveRobotsCount > 2) {
                P = 0.4;
            }
        } else if (_robot._currentEnegy <= Cynical.ENERGY_WARNING_THRESHOLD) {
            P = 0.25;
            if (_robot._aliveRobotsCount > 2) {
                P = 0.5;
            }
        } else if (_robot._currentEnegy <= Cynical.ENERGY_CRITICAL_THRESHOLD) {
            P = 0.6;
        }
        _curFirepower = _curTarget.getOptimalFirePower(P);
    }

    /** z50_0 :      .*/
    public void z50_0_calculateFineForestallingAndTurnGun() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z50_0", "     ");
        }
        if (_curTarget != null) {
            GeomVector predicted_pos;

            predicted_pos = _curTarget.calculateAveragePrediction(
                    Cynical.getBulletSpeed(_curFirepower), 2);

            _curAim.setCoords(predicted_pos.getAngle(), predicted_pos.getRadius());
            _da = Cynical.getAngleDiff(_curHeading, predicted_pos.getAngle());
        }
    }

    /** z50_1 :      . */
    public void z50_1_calculateRoughForestallingAndTurnGun() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z50_1", "     ");
        }
        if (_curTarget != null) {
            GeomVector predicted_pos;
            predicted_pos = _curTarget.calculateRoughPrediction(
                    Cynical.getBulletSpeed(Cynical.BASE_FIRE_POWER), 2);
            _curAim.setCoords(predicted_pos.getAngle(), predicted_pos.getRadius());
            _da = Cynical.getAngleDiff(_curHeading, predicted_pos.getAngle());
        }
    }

    /** z60 : . */
    public void z60_makeShot() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z60", "");
        }
        _firepower = _curFirepower;
    }

    /** z70 :    . */
    public void z70_dropTargetPathHistory() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z70", "   ");
        }
        if (_curTarget != null)
            _curTarget.resetSpeedHistory();
    }

    /** z80 :   . */
    public void z80_dropCurrentTarget() {
        if (Constants.OUTPUTS_LOGGING) {
            Logger.logOutput("z80", "  ");
        }
        _curTarget = null;
    }
}
