package newCynic;

/**
 *       ""
 */
public abstract class RadarState {

    /**  ,    */
    private String _stateName;

    /** ,  .     . */
    protected RadarState(String aStateName) {
        _stateName = aStateName;
    }

    /** ,   . */
    public String getName() {
        return _stateName;
    }

    /** ,  .   
     *        .
     */
    public abstract void processEvent(int aEvent, Radar aRadar);

    /** ,       .  
     *         .
     */
    public abstract void onEnter(Radar aRadar);

    /**  0  "" */
    private final static RadarState STATE_0_TURN_LEFT = new RadarState0();
    /**  1  "" */
    private final static RadarState STATE_1_TURN_RIGHT = new RadarState1();
    /**  2  "" */
    private final static RadarState STATE_2_TURN_STEP_RIGHT = new RadarState2();
    /**  3  "" */
    private final static RadarState STATE_3_TURN_STEP_LEFT = new RadarState3();


    /**  ,   
     *  (     ).
     */
    public static void reset(Radar aRadar) {
        aRadar.setCurrentState(STATE_0_TURN_LEFT);
    }

    /**
     *    --  .
     */
    private static void doStartLogging(int aEvent, Radar aRadar) {
        if (Constants.OBJECTS_LOGGING) {
            Logger.log("  '':");
        }
        if (Constants.A4_BEGIN_LOGGING) {
            Logger.logBegin(Constants.RADAR_AUTOMATE_NAME, aRadar.getCurrentState().getName(), aEvent);
        }
    }

    /**
     *    --  .
     */
    private static void doEndLogging(int aEvent, Radar aRadar) {
        if (Constants.A4_END_LOGGING) {
            Logger.logEnd(Constants.RADAR_AUTOMATE_NAME, aRadar.getCurrentState().getName());
        }
    }

    /**
      *   ,  .
      */
    private static void changeState(RadarState aNewState, Radar aRadar) {
        if (Constants.A4_TRANS_LOGGING) {
            Logger.logStateChange(Constants.RADAR_AUTOMATE_NAME, aNewState.getName(), aRadar.getCurrentState().getName());
        }
        aRadar.setCurrentState(aNewState);
        aNewState.onEnter(aRadar);
    }
    /**
      *  ,    aEvent  aRadar.
     *     .
      */

    public static void processIncomingEvent(int aEvent, Radar aRadar) {
        doStartLogging(aEvent, aRadar);
        aRadar.getCurrentState().processEvent(aEvent, aRadar);
        doEndLogging(aEvent, aRadar);
    }

    /** ,   0  "" */
    private static class RadarState0 extends RadarState {
        public RadarState0() {
            super("State 0");
        }

        public void processEvent(int aEvent, Radar aRadar) {
            if (aRadar.x70_scanningIsComplete() && aRadar.x80_wholePathIsLesserThan180()) {
                aRadar.z101_0_resetState();
                changeState(STATE_1_TURN_RIGHT, aRadar);
            } else if (aRadar.x70_scanningIsComplete()) {
                changeState(STATE_2_TURN_STEP_RIGHT, aRadar);
            } else {
                aRadar.z100_0_turnLeft();
            }
        }

        public void onEnter(Radar aRadar) {
            aRadar.z100_0_turnLeft();
        }
    }

    /** ,   1  "" */
    private static class RadarState1 extends RadarState {
        public RadarState1() {
            super("State 1");
        }

        public void processEvent(int aEvent, Radar aRadar) {
            if (aRadar.x70_scanningIsComplete() && aRadar.x80_wholePathIsLesserThan180()) {
                aRadar.z101_0_resetState();
                changeState(STATE_0_TURN_LEFT, aRadar);
            } else if (aRadar.x70_scanningIsComplete()) {
                changeState(STATE_3_TURN_STEP_LEFT, aRadar);
            } else {
                aRadar.z100_1_turnRight();
            }
        }

        public void onEnter(Radar aRadar) {
            aRadar.z100_1_turnRight();
        }
    }

    /** ,   2  "" */
    private static class RadarState2 extends RadarState {
        public RadarState2() {
            super("State 2");
        }

        public void processEvent(int aEvent, Radar aRadar) {
            if (aRadar.x70_scanningIsComplete()) {
                aRadar.z101_0_resetState();
            }
            changeState(STATE_0_TURN_LEFT, aRadar);
        }

        public void onEnter(Radar aRadar) {
            aRadar.z101_0_resetState();
            aRadar.z100_1_turnRight();
        }
    }

    /** ,   3  "" */
    private static class RadarState3 extends RadarState {
        public RadarState3() {
            super("State 3");
        }

        public void processEvent(int aEvent, Radar aRadar) {
            if (aRadar.x70_scanningIsComplete()) {
                aRadar.z101_0_resetState();
            }
            changeState(STATE_1_TURN_RIGHT, aRadar);

        }

        public void onEnter(Radar aRadar) {
            aRadar.z101_0_resetState();
            aRadar.z100_0_turnLeft();
        }
    }
}
