package robocode.control;

import ru.ifmo.robocode.TankManager;
import newCynic.Cynical;
import robocode.Robot;
import robocode.repository.RobotSpec;

import java.io.File;

// score: 1.0946595460614152
// ticks: 141503
// ticks/sec: 4413

public class EffTest {

    public static final Object mutex = new Object();
    private static final String ROBOTS_DIR = "robots";

    private static final int NUM_ROUNDS = 100;

    private static final int INACTIVITY_TIME = 450;
    private static final double GUN_COOLING_RATE = 0.1;

    private static final Robot myTank = new Cynical();
    private static final Robot enemyClazz = new Cynical();

    private static RobotSpecification getRobot(String clazz) {
        return new RobotSpecification(new RobotSpec(new File(clazz + ".class"), new File(ROBOTS_DIR), "", false));
    }

    private static int numTiks;
    private static long ticksTime;

    public static void main(String[] args) {

        long start = System.currentTimeMillis();

        RobocodeEngine engine = new RobocodeEngine(new RobocodeListener() {
            public void battleComplete(BattleSpecification battle, RobotResults[] results, int numTiks, long ticksTime, long robotsTime) {
                System.out.println("score: " + results[0].getScore() * 1.0 / results[1].getScore());
                System.out.println(results[0].getScore());
                System.out.println("ticks: " + numTiks);
                System.out.println("robots time: " + robotsTime / 1000);
                EffTest.numTiks = numTiks;
                EffTest.ticksTime = ticksTime;
                synchronized (mutex) {
                    mutex.notifyAll();
                }
            }
            public void battleAborted(BattleSpecification battle) {}
            public void battleMessage(String message) {
                System.out.println(message);
            }
        });

        BattlefieldSpecification bfspec = new BattlefieldSpecification();
        RobotSpecification enemy = getRobot(enemyClazz.getClass().getName());

        RobotSpecification my = getRobot(myTank.getClass().getName());
        TankManager.getInstance().setObjects(false, false);

        RobotSpecification[] rs = {my, enemy};
        BattleSpecification bspec = new BattleSpecification(NUM_ROUNDS, INACTIVITY_TIME, GUN_COOLING_RATE, bfspec, rs);
        engine.runBattle(bspec);
        engine.close();

        long dt = System.currentTimeMillis() - start;
        System.out.println("time: " + dt / 1000);
        System.out.println("ticks time: " + ticksTime / 1000);
        System.out.println("ticks/sec: " + numTiks * 1000 / ticksTime);

    }

}
