package ru.ifmo.robocode;

public class Environment {

    // my tank
    final double X, Y, V, H; // my coord, velocity and heading
    final double GH;
    final double E;
    final long tHit;//last hit time

    // target
    final double x; // rel to mine
    final double y; // rel to mine
    final double h;
    final double v;
    final double e;

    // time of last measure
    final long t;

    public final double dh;        // angle between my gun ant center of target
    public final double d;         // distance

    final double w;         // distance to nearest wall

    final double dr;        // remaining distance
    final double tr;        // remaining turn

    public Environment(double x, double y, double v, double h, long t, double X, double Y, double V, double H, double GH, double E, double e, long tHit, double w, double dr, double tr) {
        this.tr = tr;
        this.GH = GH;
        this.x = x;
        this.y = y;
        this.v = v;
        this.h = h;
        this.t = t;
        this.E = E;
        this.X = X;
        this.Y = Y;
        this.V = V;
        this.H = H;
        this.w = w;
        dh = getAngle(x, y) - GH;
        d = Math.sqrt(x * x + y * y);
        this.e = e;
        this.tHit = tHit;
        this.dr = dr;
    }

    private double getAngle(double x, double y) {
        double a = Math.atan2(y, x);
        if (a < 0) {
            a += 2 * Math.PI;
        }
        a -= Math.PI / 2;
        if (a < 0) {
            a += 2 * Math.PI;
        }
        return 2 * Math.PI - a;
    }

    public static String[] firstTerms() {
        return new String[]{"X", "Y", "x", "y", "d", "w"};
    }

    public static String[] movementTerms() {
        return new String[]{"x", "y", "dr", "tr", "w", "0.5", "0", "1", "2"};
    }

    public double[] movementValues() {
        return new double[]{x, y, dr, tr, w, .5, 0, 1, 2};
    }

    public static String[] targetingTerms() {
        return new String[]{"dh", "GH", "h", "d", "0.5", "Pi", "0", "1", "2", "10", "100", "H"};
    }

    public double[] targetingValues() {
        return new double[]{dh, GH, h, d, .5, Math.PI, 0, 1, 2, 10, 100, H};
    }

    public double[] values() {
        return new double[]{x, y, dr, tr, w, .5, 0, 1, 2, 10, dh, GH, h, d, e, E};
    }

    public static String[] terms() {
        return new String[]{"x", "y", "dr", "tr", "w", ".5", "0", "1", "2", "10", "dh", "GH", "h", "d", "e", "E"};
    }
}
