package ru.ifmo.robocode;

import ru.ifmo.robocode.gep.IntString;

import java.util.LinkedList;
import java.util.Queue;

public class Expression {
    private final int t[][];
    private final IntString ch;
    private final BasisFunc basis[];
    private final String terms[];

    public double calc(double terms[]) {
        return f(0, terms);
    }

    private double f(int i, double terms[]) {
        if (isTerm(i)) {
            return terms[ch.get(i) - basis.length];
        }
        BasisFunc func = basis[ch.get(i)];
        double args[] = new double[func.arity()];
        for (int j = 0; j < args.length; j++) {
            args[j] = f(t[i][j], terms);
        }
        return func.f(args);
    }

    private boolean isTerm(int i) {
        return ch.get(i) >= basis.length;
    }

    private void express() {
        int i = 0;
        Queue<Integer> q1 = new LinkedList<Integer>();
        q1.add(i++);
        while (!q1.isEmpty()) {
            Queue<Integer> q2 = new LinkedList<Integer>();
            for (int x : q1) {
                if (isTerm(x)) continue;
                BasisFunc func = basis[ch.get(x)];
                for (int j = 0; j < func.arity(); j++, i++) {
                    t[x][j] = i;
                    q2.add(i);
                }
            }
            q1 = q2;
        }
    }

    public Expression(IntString s, BasisFunc func[], String terms[]) {
        this.terms = terms;
        int max = 0;
        for (BasisFunc basisFunc : func) {
            max = Math.max(max, basisFunc.arity());
        }
        t = new int[s.length()][max];
        this.basis = func;
        ch = s;
        express();
    }

    public String toString() {
        return toString(0);
    }

    private String toString(int i) {
        if (isTerm(i)) {
            return terms[ch.get(i) - basis.length];
        }
        BasisFunc func = basis[ch.get(i)];
        String args[] = new String[func.arity()];
        for (int j = 0; j < args.length; j++) {
            args[j] = toString(t[i][j]);
        }
        return func.toString(args);
    }

    public String toCanonic() {
        return toCanonic(0);
    }

    public String toCanonic(int i) {
        if (isTerm(i)) {
            return terms[ch.get(i) - basis.length];
        }

        BasisFunc func = basis[ch.get(i)];
        StringBuilder sb = new StringBuilder();
        sb.append(func.s()).append("(");
        for (int j = 0; j < func.arity(); j++) {
            sb.append(toCanonic(t[i][j]));
            if (j < func.arity() - 1) {
                sb.append(", ");
            }
        }
        sb.append(")");

        return sb.toString();
    }
}
