package ru.ifmo.robocode;

import ru.ifmo.robocode.Environment;
import ru.ifmo.robocode.TankManager;
import ru.ifmo.robocode.ga.Configuration;
import ru.ifmo.robocode.ga.Genome;
import ru.ifmo.robocode.gep.Chromosome;
import ru.ifmo.robocode.gep.GepConfiguration;
import org.apache.log4j.Logger;

import java.io.FileWriter;
import java.io.IOException;

public class Main {
    private final static Logger LOG = Logger.getLogger(Main.class);

    private final static int NUM_POPUlATIONS = 100;
    private final static int POPULATION_SIZE = 50;

    public static final String BEST_FILE = "res/exp/Best.exp";
    public static final String CURRENT_FILE = "res/exp/Current.exp";

    public static void main(String[] args) throws IOException {
        showInfo();
        Configuration<Chromosome> cfg = new GepConfiguration();
        cfg.setPopulationSize(POPULATION_SIZE);
        cfg.setChromosomeLength(GepConfiguration.LEN);
        cfg.setFitness(new Fitness());
        Genome<Chromosome> genome = Genome.randomize(cfg);
        save(genome.getFittestChromosome(), true);
        double max = genome.getFittest();
        for (int i = 0; i < NUM_POPUlATIONS; i++) {
            save(genome.getFittestChromosome(), genome.getFittest() > max);
            max = Math.max(max, genome.getFittest());
            genome.evolve();
        }
        LOG.info("result: " + genome.getFittest());

    }

    private static void showInfo() {
        System.out.println("Main");
        System.out.print("enemies: ");
        for (String s : TankManager.getInstance().getEnemies()) {
            System.out.print(s + ", ");
        }
        System.out.println();
        System.out.println("population size: " + POPULATION_SIZE);
        System.out.println("num populations: " + NUM_POPUlATIONS);
        System.out.println("num rounds: " + Fitness.NUM_ROUNDS);
        System.out.println("num of chromosomes: " + GepConfiguration.NUM_EXPR);
        System.out.println("chromosome len: " + GepConfiguration.LEN);
        System.out.print("functions: ");
        for (Basis basis : Basis.values()) {
            System.out.print(basis.toString() + ", ");
        }
        System.out.println();
        System.out.print("terminals: ");
        for (String s : Environment.terms()) {
            System.out.print(s + ", ");
        }
        System.out.println();
    }

    private static void save(Chromosome c, String file) throws IOException {
        FileWriter fw = new FileWriter(file);
        c.save(fw);
        fw.close();
    }

    private static void save(Chromosome c, boolean max) throws IOException {
        save(c, CURRENT_FILE);
        if (max) {
            save(c, BEST_FILE);
        }
    }

}
